﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Input;

using DemoDataSources.WpfApp.Commands;
using DemoDataSources.Models;
using DemoDataSources.Repository;
using Infragistics.Windows.Controls;
using Infragistics.Windows.DataPresenter;
using System.Diagnostics;
using System.Reflection;

namespace DemoDataSources.WpfApp.ViewModels
{
	public class MainViewModel : ViewModelBase
	{

		private ICommand exitCommand;
		private ICommand refreshCommand;
		private ICommand alterCommand;
		private ICommand buildHeadersCommand;
		
		private BorderCrossingRepository borderCrossingRepository;
		private List<Field> fieldCollection;
		private ObservableCollection<BorderCrossing> borderCrossingCollection;
		
		public FieldLayoutSettings FieldLayoutSettings { get; set; }
		public FieldSettings FieldSettings { get; set; }

		public ObservableCollection<BorderCrossing> BorderCrossingCollection
		{ 
		
			get
			{
				return borderCrossingCollection;
			}

			set
			{

				if (borderCrossingCollection != value)
				{
					borderCrossingCollection = value;
					OnPropertyChanged("BorderCrossingCollection");
				}
			}
		
		}

		public List<Field> FieldCollection 
		{ 
		
			get
			{
				return fieldCollection;
			}

			set
			{

				if (fieldCollection != value)
				{
					fieldCollection = value;
					OnPropertyChanged("FieldCollection");
				}
			}
		
		}

		public ICommand ExitCommand
		{
			get
			{
				if (exitCommand == null)
				{
					exitCommand = new DelegateCommand(Exit);
				}
				return exitCommand;
			}
		}

		public ICommand BuildHeadersCommand
		{
			get
			{
				if (buildHeadersCommand == null)
					buildHeadersCommand = new DelegateCommand(BuildHeaders);
				return buildHeadersCommand;
			}
		}
		
		#region Constructor
		public MainViewModel()
		{

			FieldLayoutSettings = new Infragistics.Windows.DataPresenter.FieldLayoutSettings();
			FieldSettings = new Infragistics.Windows.DataPresenter.FieldSettings();
			FieldCollection = new List<Field>();

			InitDataSource();
			InitXamDataGrid();

		}

		private void InitDataSource() 
		{
			borderCrossingRepository = new BorderCrossingRepository();
			BorderCrossingCollection = new ObservableCollection<BorderCrossing>(borderCrossingRepository.Get());
			OnPropertyChanged("BorderCrossingCollection");
		}

		private void InitXamDataGrid() 
		{

			FieldLayoutSettings = new FieldLayoutSettings
			{
				AutoGenerateFields = false,
				AutoArrangeCells = AutoArrangeCells.Never,
				FixedFieldUIType = FixedFieldUIType.Splitter,
				AllowFieldMoving = AllowFieldMoving.No,
				HeaderPlacementInGroupBy = HeaderPlacementInGroupBy.OnTopOnly,
				SelectionTypeCell = SelectionType.Extended,
				DataRecordSizingMode = DataRecordSizingMode.SizedToContentAndIndividuallySizable,
				DataErrorDisplayMode = DataErrorDisplayMode.ErrorIcon,
				FilterUIType = FilterUIType.LabelIcons,
				GroupByExpansionIndicatorVisibility = Visibility.Collapsed,
				SummaryDescriptionVisibility = Visibility.Collapsed,
				GroupBySummaryDisplayMode = GroupBySummaryDisplayMode.Default,
				SelectionTypeField = SelectionType.Extended,
				SupportDataErrorInfo = SupportDataErrorInfo.RecordsOnly,
				AllowClipboardOperations = AllowClipboardOperations.All,
				CopyFieldLabelsToClipboard = false,
			};

			FieldSettings = new FieldSettings
			{
				AllowFixing = AllowFieldFixing.Near,
				AllowResize = true,
				LabelClickAction = LabelClickAction.SelectField,
				CellClickAction = CellClickAction.SelectCell,
				AutoSizeOptions = FieldAutoSizeOptions.All,
				AutoSizeScope = FieldAutoSizeScope.RecordsInView,
			};

			BuildHeaders();

		}

		#endregion

		public string VersionMethod()
		{
			Assembly assem = Assembly.GetExecutingAssembly();
			AssemblyName assemName = assem.GetName();
			Version ver = assemName.Version;
			return assemName.Name + ", Version " + ver.ToString();
		}

		private void BuildHeaders()
		{

			var bottom = 2;
			var middle = 1;
			var top = 0;

			Field field;
			UnboundField unboundField;

			var hiddenStyle = new Style(typeof(CellValuePresenter));
			hiddenStyle.Setters.Add(new Setter(UIElement.VisibilityProperty, Visibility.Collapsed));

			var fields = new List<Field>();

			// Id, 0
			field = new Field
			{
				Name = "Id",
				Visibility = Visibility.Collapsed,
				Column = 0,
				Row = bottom,
				
			};
			fields.Add(field);

			// State 1
			field = new Field
			{
				Name = "State",
				Visibility = Visibility.Visible,
				Column = 1,
				Row = bottom,
			};
			fields.Add(field);
		  	
			// Port 2
			field = new Field
			{
				Name = "Port",
				Visibility = Visibility.Visible,
				Column = 2,
				Row = bottom,
			};
			fields.Add(field);
				
			// Year 3
			field = new Field
			{
				Name = "Year",
				Visibility = Visibility.Visible,
				Column = 3,
				Row = bottom,
			};
			fields.Add(field);
			
			// Month 4
			field = new Field
			{
				Name = "Month",
				Visibility = Visibility.Visible,
				Column = 4,
				Row = bottom,
			};
			fields.Add(field);

			unboundField = new UnboundField
			{
				Row = top,
				Column = 5,
				ColumnSpan = 11,
				Label = "Vehicles",
				Settings =
				{
					CellMaxHeight = 0,
					AllowGroupBy = false,
					CellValuePresenterStyle = hiddenStyle,
				}
			};
			fields.Add(unboundField);

			unboundField = new UnboundField
			{
				Row = middle,
				Column = 5,
				ColumnSpan = 3,
				Label = "Trucks",
				Settings =
				{
					CellMaxHeight = 0,
					AllowGroupBy = false,
					CellValuePresenterStyle = hiddenStyle,
				}

			};
			fields.Add(unboundField);

			//  Trucks 5
			field = new Field
			{
				Name = "Trucks",
				Label = "All Trucks",
				Visibility = Visibility.Visible,
				Column = 5,
				Row = bottom,
			};
			fields.Add(field);
			
			//  LoadedTrucks 6
			field = new Field
			{
				Name = "LoadedTrucks",
				Label = "Loaded",
				Visibility = Visibility.Visible,
				Column = 6,
				Row = bottom,
			};
			fields.Add(field);

			//  EmptyTrucks 7
			field = new Field
			{
				Name = "EmptyTrucks",
				Label = "Empty",
				Visibility = Visibility.Visible,
				Column = 7,
				Row = bottom,
			};
			fields.Add(field);

			unboundField = new UnboundField
			{
				Row = middle,
				Column = 8,
				ColumnSpan = 4,
				Label = "Trains",
				Settings =
				{
					CellMaxHeight = 0,
					AllowGroupBy = false,
					CellValuePresenterStyle = hiddenStyle,
				}

			};
			fields.Add(unboundField);

			//  Trains 8
			field = new Field
			{
				Name = "Trains",
				Label = "All Trains",
				Visibility = Visibility.Visible,
				Column = 8,
				Row = bottom,
			};
			fields.Add(field);

			//  LoadedRailContainers 9
			field = new Field
			{
				Name = "LoadedRailContainers",
				Label = "Loaded Containers",
				Visibility = Visibility.Visible,
				Column = 9,
				Row = bottom,
			};
			fields.Add(field);

			//  EmptyRailContainers 10
			field = new Field
			{
				Name = "EmptyRailContainers",
				Label = "Empty Containers",
				Visibility = Visibility.Visible,
				Column = 10,
				Row = bottom,
			};
			fields.Add(field);

			//  TrainPassengers 11
			field = new Field
			{
				Name = "TrainPassengers",
				Label = "Passengers",
				Visibility = Visibility.Visible,
				Column = 11,
				Row = bottom,
			};
			fields.Add(field);

			unboundField = new UnboundField
			{
				Row = middle,
				Column = 12,
				ColumnSpan = 2,
				Label = "Buses",
				Settings =
				{
					CellMaxHeight = 0,
					AllowGroupBy = false,
					CellValuePresenterStyle = hiddenStyle,
				}

			};
			fields.Add(unboundField);

			//  Buses 12
			field = new Field
			{
				Name = "Buses",
				Label = "All",
				Visibility = Visibility.Visible,
				Column = 12,
				Row = bottom,
			};
			fields.Add(field);

			//  BusPassengers 13
			field = new Field
			{
				Name = "BusPassengers",
				Label = "Passengers",
				Visibility = Visibility.Visible,
				Column = 13,
				Row = bottom,
			};
			fields.Add(field);

			unboundField = new UnboundField
			{
				Row = middle,
				Column = 14,
				ColumnSpan = 2,
				Label = "Personal Vehicles",
				Settings =
				{
					CellMaxHeight = 0,
					AllowGroupBy = false,
					CellValuePresenterStyle = hiddenStyle,
				}

			};
			fields.Add(unboundField);
			
			//  PersonalVehicles 14
			field = new Field
			{
				Name = "PersonalVehicles",
				Label = "All",
				Visibility = Visibility.Visible,
				Column = 14,
				Row = bottom,
			};
			fields.Add(field);
	
			//  PassengersPersonalVehicles 15
			field = new Field
			{
				Name = "PassengersPersonalVehicles",
				Label = "Passengers",
				Visibility = Visibility.Visible,
				Column = 15,
				Row = bottom,
			};
			fields.Add(field);

			//  Pedestrians 16
			field = new Field
			{
				Name = "Pedestrians",
				Visibility = Visibility.Visible,
				Column = 16,
				Row = bottom,
			};
			fields.Add(field);

			FieldCollection = fields;
		}
		
		private void Exit()
		{
			Application.Current.Shutdown();
		}

	}

}
